/**
 * OW2 Util
 * Copyright (C) 2008-2010 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: BasicCluePoolStatsFillerPoolListener.java 5502 2010-05-26 09:06:11Z benoitf $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.pool.impl.enhanced.impl.listener.statistics.clue;

import java.util.ArrayList;
import java.util.List;

import org.ow2.util.pool.impl.enhanced.api.IWaitControl;
import org.ow2.util.pool.impl.enhanced.api.PoolException;
import org.ow2.util.pool.impl.enhanced.api.listener.statistics.clue.ICluePoolStatsListener;
import org.ow2.util.pool.impl.enhanced.impl.listener.statistics.BasicPoolStats;
import org.ow2.util.pool.impl.enhanced.impl.listener.statistics.BasicPoolStatsFillerPoolListener;

/**
 * @author Gael Lalire
 * @param <E> pool item type
 * @param <C> clue type
 */
public class BasicCluePoolStatsFillerPoolListener<E, C> extends BasicPoolStatsFillerPoolListener<E> implements
        ICluePoolStatsListener<E, C, BasicPoolStats> {

    private BasicPoolStats basicPoolStats;

    private boolean countPutAsSentBack;

    /**
     * List to keep the reference of get/put when we don't use the special flag countPutAsSentBack flag.
     * When we call get method, the instance should be flagged as a busy instance until it is removed.
     */
    private List<String> managedInstances = new ArrayList<String>();


    public BasicCluePoolStatsFillerPoolListener() {
        this(true);
    }

    /**
     * Hack to allow not resetable clue (the clue manager should be reviewed to manage it).
     */
    @Deprecated
    public BasicCluePoolStatsFillerPoolListener(final boolean countPutAsSentBack) {
        super();
        this.countPutAsSentBack = countPutAsSentBack;
        basicPoolStats = getBasicPoolStats();
    }

    public void getMethodCalled(final C clue, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() + 1);
        }
    }

    public void getMethodFailed(final PoolException poolException, final C clue, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() - 1);
        }
    }

    public void getMethodReturned(final E result, final C clue, final IWaitControl timeout) {
        synchronized (basicPoolStats) {
            if (!countPutAsSentBack) {
                String hash = String.valueOf(System.identityHashCode(result));
                // Only increment if there was no previous get on this instance
                if (!managedInstances.contains(hash)) {
                    managedInstances.add(hash);
                    basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() + 1);
                 }
            } else {
                basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() + 1);
            }
            basicPoolStats.setWaiters(basicPoolStats.getWaiters() - 1);
        }
    }

    @Override
    public void putMethodReturned(final E poolItem) {
        synchronized (basicPoolStats) {
            // The instance will be kept if countPutAsSentBack is false
            if (countPutAsSentBack) {
                basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() - 1);
            }
        }
    }

    @Override
    public void removeMethodReturned(final E poolItem) {
        synchronized (basicPoolStats) {
            if (!countPutAsSentBack) {
                managedInstances.remove(String.valueOf(System.identityHashCode(poolItem)));
            }
            basicPoolStats.setNotSendedBack(basicPoolStats.getNotSendedBack() - 1);
        }
    }

}
